//
// File:    Logger.java
// Author:  Paul Marlow
// Date:    July 23, 2001
//


import java.io.*;
import java.net.*;
import java.util.StringTokenizer;


//***************************************************************************
//
//	This is main object class
//
//***************************************************************************
class ClientServerComm extends Thread
{
  //===========================================================================
  // Private members
  private DatagramSocket listen_socket;  // Initial socket to listen on
  private LogMemory log_memory;
  private boolean stopthread = false;

  //===========================================================================
  // Initialization member functions

  //---------------------------------------------------------------------------
  // This constructor opens socket connection
  public ClientServerComm(DatagramSocket listen, LogMemory memory)
  {
    listen_socket = listen;
    log_memory = memory;
    start();
  }

  //---------------------------------------------------------------------------
  // This destructor closes socket to server and client
  public void finalize()
  {
  }


  //===========================================================================
  // Public member functions

  public synchronized void initiateStop()
  {
    stopthread = true;
  }

  public synchronized boolean stopThread()
  {
    return stopthread;
  }

  public void run()
  {
    byte[] buffer = new byte[log_memory.getPacketSize()];
    DatagramPacket packet = new DatagramPacket(buffer, log_memory.getPacketSize());

    try {
      listen_socket.receive(packet);
    } catch (IOException e) {
      System.err.println("Socket receiving error " + e);
    }

    // Route packet to actual destination
    String record = (new String(buffer,0)).trim();

    // Packet received - check to see if server and/or client port has been
    // initialized, and intialize if it has not been
    log_memory.setClientHost(packet.getAddress());
    log_memory.setClientPort(packet.getPort());
    log_memory.parseClientInit(record);

    send(record);
    //log_memory.log.addRecord(record);

    // Continually loop receiving/sending messages until thread is stopped
    while (!stopThread()) {
      record = receive();
      send(record);
      log_memory.log.addRecord(record);
    }
  }

  //===========================================================================
  // Private member functions

  //---------------------------------------------------------------------------
  // This function sends via socket message to the server/client
  private void send(String message)
  {
    byte[] buffer = new byte[log_memory.getPacketSize()];
    message.getBytes(0, message.length(), buffer, 0);
    DatagramPacket packet = new DatagramPacket(buffer, log_memory.getPacketSize(), log_memory.server_host, log_memory.server_port);
    try {
      log_memory.server_socket.send(packet);
    } catch (IOException e) {
      System.err.println("Socket sending error " + e);
    }
  }

  //---------------------------------------------------------------------------
  // This function waits for new message from server/client
  private String receive() 
  {
    byte[] buffer = new byte[log_memory.getPacketSize()];
    DatagramPacket packet = new DatagramPacket(buffer, log_memory.getPacketSize());

    try {
      log_memory.client_socket.receive(packet);
    } catch (IOException e) {
      System.err.println("Socket receiving error " + e);
    }

    return (new String(buffer,0)).trim();
  }
}
