//
// File:    LogMemory.java
// Author:  Paul Marlow
// Date:    October 15, 2001
//


import java.io.*;
import java.net.*;
import java.util.StringTokenizer;

//***************************************************************************
//
//	This is main object class
//
//***************************************************************************
class LogMemory
{
  //===========================================================================
  // Private members
  public InetAddress server_host;  // Server host address
  public InetAddress client_host;  // Client host address
  public int server_port = 0;  // Server port number for communication
  public int client_port = 0;  // Client port number for communication
  public DatagramSocket server_socket;  // Socket for communicating with Server
  public DatagramSocket client_socket;  // Socket for communicating with Client
  public Logger log;  // Log writing
  private String logfile; 
  private String playername; // Name of the player - teamname + player num
  private String teamname; // Name of the team the player plays for
  private String playernum; // Uniform number of the player
  private int packet_size;

  //===========================================================================
  // Initialization member functions

  //---------------------------------------------------------------------------
  // This constructor opens socket for connection with server
  public LogMemory(InetAddress s_host, int s_port)
  {
    server_host = s_host;
    server_port = s_port;
    playername = "";
    teamname = "";
    playernum = "";
    logfile = "";
    client_port = 0;
    client_host = null;
    log = null;
    packet_size = 4096;
    try {
      server_socket = new DatagramSocket();
      client_socket = new DatagramSocket();
    } catch (SocketException se) {
      System.err.println("Unable to create sockets for communication: " + se);
      System.exit(0);
    }
  }
																 
  //---------------------------------------------------------------------------
  // This destructor closes socket to server and client
  public void finalize()
  {
  }


  //===========================================================================
  // Public member functions

  public synchronized boolean checkClient()
  {
    return (client_port > 0);
  }

  public synchronized void setClientPort(int port)
  {
    client_port = port;
  }

  public synchronized void setClientHost(InetAddress host)
  {
    client_host = host;
  }

  public synchronized int getClientPort()
  {
    return client_port;
  }

  public synchronized InetAddress getClientHost()
  {
    return client_host;
  }

  public synchronized boolean checkServer()
  {
    return (server_port != 6000);
  }

  public synchronized void setServerPort(int port)
  {
    server_port = port;
  }

  public synchronized void setServerHost(InetAddress host)
  {
    server_host = host;
  }

  public synchronized int getServerPort()
  {
    return server_port;
  }

  public synchronized InetAddress getServerHost()
  {
    return server_host;
  }

  public synchronized void setPlayerNumber(String p_num)
  {
    playernum = p_num;
    playername = teamname + "_" + playernum;
    createOutputFile();
  }

  public synchronized void setTeamname(String t_name)
  {
    teamname = t_name;
    playername = teamname + "_" + playernum;
    createOutputFile();
  }

  public synchronized String getPlayerName()
  {
    return playername;
  }

  public synchronized int getPacketSize()
  {
    return packet_size;
  }

  public synchronized void setPacketSize(int p_size)
  {
    packet_size = p_size;
  }

  // Parse init commands from client and server to obtain Teamname and
  // Player Number (Uniform Number) information about the player

  public synchronized void parseClientInit(String init)
  {
    double version_number = 3.00;
    StringTokenizer tokenizer = new StringTokenizer(init,"() ");
		
    // Check for "init"
    if(tokenizer.nextToken().compareTo("init") == 0)
    {
      setTeamname(tokenizer.nextToken());

      while(tokenizer.hasMoreTokens())
      {
        if(tokenizer.nextToken().compareTo("version") == 0)
        {
          // Get version number used for communication between client and server.
          version_number = Double.parseDouble(tokenizer.nextToken());
        }
      }

      // Depending on the version number, the packet message size must be changed.
      if (version_number >= 6.00) {
        setPacketSize(4096);
      } else {
        setPacketSize(1024);
      }
    }
  }

  public synchronized void parseServerInit(String init)
  {
    StringTokenizer tokenizer = new StringTokenizer(init,"() ");
		
    // Check for "init"
    if(tokenizer.nextToken().compareTo("init") == 0)
    {
	// Skip by side
      tokenizer.nextToken();
      // Set player number
      setPlayerNumber(tokenizer.nextToken());
    }
  }

  public String getFilename()
  {
    return logfile;
  }

  //===========================================================================
  // Protected member functions

  //===========================================================================
  // Private member functions

  private void createOutputFile()
  {
    if (teamname.compareTo("") != 0 && playernum.compareTo("") != 0 && log == null) {
      logfile = playername + ".lsf";
      log = new Logger(logfile);
    }
  }
}
