
//
// File:    LogServer.java
// Author:  Paul Marlow
// Date:    July 27, 2001
//


import java.io.*;
import java.net.*;
import java.util.StringTokenizer;
//import java.util.LinkedList;
import java.util.Vector;


//***************************************************************************
//
//	This is main object class
//
//***************************************************************************
class LogServer
{
  //===========================================================================
  // Private members
  private InetAddress s_host;  // Server host address
  private InetAddress c_host;  // Client host address
  private int s_init_port = 6000;  // Initialization port of Soccer Server
  private int s_port = 0;  // Server port number for communication
  private int c_port = 0;  // Client port number for communication
  private Vector communication_list; // List of Server and Client Communications
  private DatagramSocket listen_socket;

  //===========================================================================
  // Initialization member functions

  //---------------------------------------------------------------------------
  // The main appllication function.
  // Command line format:
  //
  // LogServer [-parameter value]
  //
  // Parameters:
  //
  //   s_host (default "localhost")
  //     The server host name can either be a machine name
  //
  //   s_port (default 6000)
  //     Port number for communication with server
  //
  // 
  public static void main(String a[]) throws IOException
  {
    String s_hostName = new String("");
    int s_portNum = 6000;

    try
    {                                                                                
      // First look for parameters
      for( int c = 0 ; c < a.length ; c += 2 )
      {
        if( a[c].compareTo("-s_host") == 0 )
        {
          s_hostName = a[c+1];
        }
        else if( a[c].compareTo("-s_port") == 0 )
        {
          s_portNum = Integer.parseInt(a[c+1]);
        }
        else
        {
          throw new Exception();
        }
      }
    }
    catch(Exception e)
    {
      System.err.println("");
      System.err.println("USAGE: java LogServer [-parameter value]");
      System.err.println("");
      System.err.println("    Parameters  value        default");
      System.err.println("   ------------------------------------");
      System.err.println("    s_host      host_name    localhost");
      System.err.println("    s_port      port_number  6000");
      System.err.println("");
      System.err.println("    Example:");
      System.err.println("      java LogServer -s_host 127.0.0.1 -s_port 6000");
      return;
    }

    System.out.println("Initializing LogServer ...");
    LogServer logserver = new LogServer(InetAddress.getByName(s_hostName), s_portNum);

    // enter main loop
    logserver.mainLoop();                                               
  }  

  //---------------------------------------------------------------------------
  // This constructor opens socket for connection with server
  public LogServer(InetAddress host, int port)
  {
    s_host = host;
    s_init_port = port;
    communication_list = new Vector();
    try {
      listen_socket = new DatagramSocket(7000);
    } catch (SocketException se) {
      System.err.println("Unable to initialize socket connection on port 7000...aborting.");
      System.exit(0);
    }
  }
																 
  //---------------------------------------------------------------------------
  // This destructor closes socket to server and client
  public void finalize()
  {
  }


  //===========================================================================
  // Public member functions


  //===========================================================================
  // Protected member functions

  //---------------------------------------------------------------------------
  // This is main loop for the LogServer
  protected void mainLoop()
  {
    LogMemory log_memory;

    ClientServerComm clientserver = null;
    ServerClientComm serverclient = null;

    // Start a communcation thread to for initialization between client and server
    System.out.println("Starting LogServer ...");

    // Press ENTER key to stop
    System.out.println("\nAt any time, press ENTER key to stop.\n");
    boolean done = false;

    while (!done) {
      try {
        log_memory = new LogMemory(s_host, s_init_port);
        serverclient = new ServerClientComm(log_memory);
        clientserver = new ClientServerComm(listen_socket, log_memory);

        // Loop until first socket connection has been made, and client port
        // has been determined in order to allow communication to the client
        System.out.println("Awaiting connection ... ");
        while (!log_memory.checkClient() && !done) {
          done = (System.in.available() > 0);
        }

        communication_list.add(clientserver);
        communication_list.add(serverclient);

        while (!log_memory.checkServer() && !done) {
          done = (System.in.available() > 0);
        }

        if (!done)
        {
          System.out.println("Logging initiated for: " + log_memory.getPlayerName() + " -> " + log_memory.getFilename());
        }
      }
      catch (IOException ioe) {
        System.err.println("Input error: " + ioe);
      }
    }

    System.out.println("Initiating stop of communication threads.");
    ClientServerComm temp_clientserver = null;
    ServerClientComm temp_serverclient = null;
    for (int i = 0; i < communication_list.size(); i+=2) {
      temp_clientserver = (ClientServerComm)(communication_list.elementAt(i));
      temp_serverclient = (ServerClientComm)(communication_list.elementAt(i+1));
      temp_clientserver.initiateStop();
      temp_serverclient.initiateStop();
    }
    System.exit(0);
  }

  //===========================================================================
  // Private member functions

}
